<?php
/**
 * All-in-one Performance Accelerator plugin file.
 *
 * Copyright (C) 2010-2020, Smackcoders Inc - info@smackcoders.com
 */

namespace Smackcoders\AIOACC;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}
if ( ! defined( 'SAVEQUERIES' ) ) {
	define( 'SAVEQUERIES', true );
}

if ( SAVEQUERIES && property_exists( $GLOBALS['wpdb'], 'save_queries' ) ) {
	$GLOBALS['wpdb']->save_queries = true;
}

	
class QueryInfo
{
	protected static $instance = null,$plugin;
    public $db_objects = array();
    public $id         = 'db_queries';
   
	public function __construct()
	{
       
	}

	
	public static function getInstance() {
      
		if ( null == self::$instance ) {
			self::$instance = new self;
			self::$plugin = Plugin::getInstance();
			self::$instance->doHooks();
		}
		return self::$instance;
	}

	
	public function doHooks(){
       
        add_action('wp_ajax_get_query_informations', array($this,'get_query_info'));
        add_action('wp_ajax_get_query_selected_tab', array($this,'get_query_selected_tab'));
        $request_action = isset($_REQUEST['action']) ? $_REQUEST['action'] : '';
		$request_page = isset($_REQUEST['page']) ? $_REQUEST['page'] : '';
		//if($request_page == 'all-in-one-performance-accelerator'){
            add_action ( 'wp_after_admin_bar_render',array($this,'smack_db_queries'), 999 );  
		//} 
    }

    function smack_db_queries () {
        $this->db_objects = apply_filters( 'qm/collect/db_objects', array(
			'$wpdb' => $GLOBALS['wpdb'],
		) );
        update_option('smack_db_queries',$this->db_objects);  
        $this->smack_query_view($this->db_objects);  
    }

    public function get_query_selected_tab(){
        $tab = sanitize_text_field($_POST['tab']);
        if($tab === 'undefined'){
            $tab_value = get_option('smack_query_tab');
            if(empty($tab_value)){
                $tab_name = 'component';
                update_option('smack_query_tab',$tab_name);
            }else{
                update_option('smack_query_tab',$tab_value);
            }
        }else{
            update_option('smack_query_tab',$tab);
        }
        $tab_address = get_option('smack_query_tab');
        $result['tab'] = $tab_address;
		$result['success'] = true;
		echo wp_json_encode($result);
		wp_die();
    }

    public function get_query_info(){
       
        $this->data['total_qs']   = 0;
		$this->data['total_time'] = 0;
		$this->data['errors']     = array();
        $this->db_objects = get_option('smack_db_queries');
        foreach ( $this->db_objects as $name => $db ) {
			if ( is_a( $db, 'wpdb' ) ) {
				$response=$this->process_db( $name, $db );
                $component_times=$this->query_status($response['component_times']);
                $caller_times=$this->caller_status($response['times']);
                $select_chart_report= $this->caller_report_status($response['times']);
                $time_chart_report= $this->chart_report_status($response['component_times']);
			} else {
				unset( $this->db_objects[ $name ] );
			}
		}

        echo wp_json_encode(['response' => ['time_chart_report' => $time_chart_report, 'plugin_query_details' => $component_times,'caller_details' => $caller_times,'caller_query_details'=>$select_chart_report ],'status' => 200, 'success' => true]);
        wp_die();  
    }

    public function smack_query_view($queryss){
        foreach ( $this->db_objects as $name => $db ) {
			if ( is_a( $db, 'wpdb' ) ) {
				$response=$this->process_db( $name, $db );
            }
        }
    }

    public function process_db($id,$db){
        global $EZSQL_ERROR, $wp_the_query;
      
              //  With SAVEQUERIES defined as false, `wpdb::queries` is empty but `wpdb::num_queries` is not.
                if ( empty( $db->queries ) ) {
                    $this->data['total_qs'] += $db->num_queries;
                    return;
                }
        
                $rows       = array();
                $types      = array();
                $total_time = 0;
                $has_result = false;
                $has_trace  = false;
                $i          = 0;
                $request    = trim( $wp_the_query->request );
               
                if ( method_exists( $db, 'remove_placeholder_escape' ) ) {
                    $request = $db->remove_placeholder_escape( $request );
                }
                foreach ( (array) $db->queries as $query ) {
                    # @TODO: decide what I want to do with this:
                    if ( false !== strpos( $query[2], 'wp_admin_bar' ) and !isset( $_REQUEST['qm_display_admin_bar'] ) ) { // phpcs:ignore
                        continue;
                    }
                    
                    $sql        = $query[0];
                    $ltime      = $query[1];
                    $stack      = $query[2];
                    $has_start  = isset( $query[3] );
                    $has_trace  = isset( $query['trace'] );
                    $has_result = isset( $query['result'] );
        
                    if ( $has_result ) {
                        $result = $query['result'];
                    } else {
                        $result = null;
                    }
        
                    $total_time += $ltime;
                  
                    if ( $has_trace ) {
        
                        $trace       = $query['trace'];
                       
                        $component   = $query['trace']->get_component();
                      
                        $caller      = $query['trace']->get_caller();
                        $caller_name = $caller['display'];
                        $caller      = $caller['display'];
        
                    } else {
        
                        $trace     = null;
                        $component = null;
                        $callers   = explode( ',', $stack );
                        $caller    = trim( end( $callers ) );
        
                        if ( false !== strpos( $caller, '(' ) ) {
                            $caller_name = substr( $caller, 0, strpos( $caller, '(' ) ) . '()';
                        } else {
                            $caller_name = $caller;
                        }
                    }
        
                    $sql  = trim( $sql );
                    $type = self::getting_queries_type( $sql );
        
                    $this->type_log( $type );
                    $this->caller_log( $caller_name, $ltime, $type );
        
                    $this->dupe_log( $sql, $i );
        
                    if ( $component ) {
                        $this->component_log( $component, $ltime, $type );
                    }
        
                    if ( ! isset( $types[ $type ]['total'] ) ) {
                        $types[ $type ]['total'] = 1;
                    } else {
                        $types[ $type ]['total']++;
                    }
        
                    if ( ! isset( $types[ $type ]['callers'][ $caller ] ) ) {
                        $types[ $type ]['callers'][ $caller ] = 1;
                    } else {
                        $types[ $type ]['callers'][ $caller ]++;
                    }
        
                    $is_main_query = ( $request === $sql && ( false !== strpos( $stack, ' WP->main,' ) ) );
        
                    $row = compact( 'caller', 'caller_name', 'sql', 'ltime', 'result', 'type', 'component', 'trace', 'is_main_query' );
        
                    if ( ! isset( $trace ) ) {
                        $row['stack'] = $stack;
                    }
        
                    if ( is_wp_error( $result ) ) {
                        $this->data['errors'][] = $row;
                    }
        
                    if ( self::is_expensive( $row ) ) {
                        $this->data['expensive'][] = $row;
                    }
        
                    $rows[ $i ] = $row;
                    $i++;
        
                }
        
                if ( '$wpdb' === $id && ! $has_result && ! empty( $EZSQL_ERROR ) && is_array( $EZSQL_ERROR ) ) {
                    // Fallback for displaying database errors when wp-content/db.php isn't in place
                    foreach ( $EZSQL_ERROR as $error ) {
                        $row = array(
                            'caller'      => null,
                            'caller_name' => null,
                            'stack'       => '',
                            'sql'         => $error['query'],
                            'ltime'       => 0,
                            'result'      => new WP_Error( 'qmdb', $error['error_str'] ),
                            'type'        => '',
                            'component'   => false,
                            'trace'       => null,
                            'is_main_query' => false,
                        );
                        $this->data['errors'][] = $row;
                    }
                }
        
                $total_qs = count( $rows );
        
                $this->data['total_qs']   += $total_qs;
                $this->data['total_time'] += $total_time;
              
                $has_main_query = wp_list_filter( $rows, array(
                    'is_main_query' => true,
                ) );

                $this->data['dbs'][ $id ] = (object) compact( 'rows', 'types', 'has_result', 'has_trace', 'total_time', 'total_qs', 'has_main_query' );
                update_option('data_db',$this->data);
                return $this->data;
            
    }

    public static function getting_queries_type( $sql ) {
		// Trim leading whitespace and brackets
		$sql = ltrim( $sql, ' \t\n\r\0\x0B(' );

		if ( 0 === strpos( $sql, '/*' ) ) {
			// Strip out leading comments such as `/*NO_SELECT_FOUND_ROWS*/` before calculating the query type
			$sql = preg_replace( '|^/\*[^\*/]+\*/|', '', $sql );
		}

		$words = preg_split( '/\b/', trim( $sql ), 2, PREG_SPLIT_NO_EMPTY );
		$type = strtoupper( $words[0] );

		return $type;
	}

    protected function type_log( $type ) {

		if ( isset( $this->data['types'][ $type ] ) ) {
			$this->data['types'][ $type ]++;
		} else {
			$this->data['types'][ $type ] = 1;
		}

	}

    protected function caller_log( $caller, $ltime, $type ) {

		if ( ! isset( $this->data['times'][ $caller ] ) ) {
			$this->data['times'][ $caller ] = array(
				'caller' => $caller,
				'ltime'  => 0,
				'types'  => array(),
			);
		}

		$this->data['times'][ $caller ]['ltime'] += $ltime;

		if ( isset( $this->data['times'][ $caller ]['types'][ $type ] ) ) {
			$this->data['times'][ $caller ]['types'][ $type ]++;
		} else {
			$this->data['times'][ $caller ]['types'][ $type ] = 1;
		}

	}

    protected function dupe_log( $sql, $i ) {

		$sql = str_replace( array( "\r\n", "\r", "\n" ), ' ', $sql );
		$sql = str_replace( array( "\t", '`' ), '', $sql );
		$sql = preg_replace( '/ +/', ' ', $sql );
		$sql = trim( $sql );
		$sql = rtrim( $sql, ';' );
		$this->data['dupes'][ $sql ][] = $i;

	}

    protected function component_log( $component, $ltime, $type ) {
            if ( ! isset( $this->data['component_times'][ $component->name ] ) ) {
                $this->data['component_times'][ $component->name ] = array(
                    'component' => $component->name,
                    'ltime'     => 0,
                    'types'     => array(),
                );
            }
            $this->data['component_times'][ $component->name ]['ltime'] += $ltime;
            if ( isset( $this->data['component_times'][ $component->name ]['types'][ $type ] ) ) {
                $this->data['component_times'][ $component->name ]['types'][ $type ]++;
            } else {
                $this->data['component_times'][ $component->name ]['types'][ $type ] = 1;
            }
    
        }

        public static function is_expensive( array $row ) {
            return $row['ltime'] > 0.05;
        }

        public function chart_report_status($component){
            $name = array_column($component, 'component');
            $time = array_column($component, 'ltime');
            $result = array_combine($name, $time);
            return $result;
            
        }

        public function caller_report_status($select){
            $name = array_column($select, 'caller');
            $time = array_column($select, 'ltime');
            $result = array_combine($name, $time);
            return $result;
        }
   
        public function query_status($plugins){
            $result_array = [];
            $temp = 0;
             foreach($plugins as $plugin){
                $result_array[$temp]['plugin'] = $plugin['component'];
                $result_array[$temp]['ltime'] = $plugin['ltime'];
                $result_array[$temp]['SELECT'] = isset($plugin['types']['SELECT']) ? $plugin['types']['SELECT'] : null ;
                $result_array[$temp]['INSERT'] = isset($plugin['types']['CREATE']) ? $plugin['types']['CREATE'] : null ;
                $result_array[$temp]['UPDATE'] = isset($plugin['types']['ALTER']) ? $plugin['types']['ALTER'] : null ;
                $result_array[$temp]['SHOW'] = isset($plugin['types']['SHOW']) ? $plugin['types']['SHOW'] : null ;
        
                $temp++;
             }
             return $result_array;  
        }

        public function caller_status($callers){
            $caller_array = [];
            $temp = 0;
             foreach($callers as $caller){
                $caller_array[$temp]['caller'] = $caller['caller'];
                $caller_array[$temp]['ltime'] = $caller['ltime'];
                $caller_array[$temp]['SELECT'] = isset($caller['types']['SELECT']) ? $caller['types']['SELECT'] : null ;
                $caller_array[$temp]['INSERT'] = isset($caller['types']['CREATE']) ? $caller['types']['CREATE'] : null ;
                $caller_array[$temp]['UPDATE'] = isset($caller['types']['ALTER']) ? $caller['types']['ALTER'] : null ;
                $caller_array[$temp]['SHOW'] = isset($caller['types']['SHOW']) ? $caller['types']['SHOW'] : null ;
        
                $temp++;
             }
            
             return $caller_array;  
        }



}