<?php
/**
 * All-in-one Performance Accelerator plugin file.
 *
 * Copyright (C) 2010-2020, Smackcoders Inc - info@smackcoders.com
 */

namespace Smackcoders\AIOACC;

if ( ! defined( 'ABSPATH' ) ) {
	die;
}

class SmackQuery {

	private $display = true;
	private $starttime, $servertime, $css, $ajax;
	protected static $instance = null,$plugin;
	public $query_row = 0;
	public static function getInstance() {
      
		if ( null == self::$instance ) {
			self::$instance = new self;
			self::$plugin = Plugin::getInstance();
			self::$instance->doHooks();
		}
		return self::$instance;
	}

	public function doHooks(){
		
		
	}

	function __construct() {
		$css='.query-dd{display:block;line-height:24px;background-color:#c4c4c4;color:#000;font-family:Helvetica,Arial,sans-serif;font-size:18px;text-align:center}.query-bs{margin:0 45px}.query-ss{margin:0 4px;border-left:5px solid #999}#query_dd{position:fixed;left:50%;bottom:0;z-index:9999998;margin-left:-160px;width:475px;height:26px;white-space:nowrap;overflow:hidden;background-color:rgb(159 188 253 / 50%);border-radius:6px;margin-bottom:21px;padding:2px}';
		$this->css = '<style type="text/css" scoped>' . str_replace(array("\r","\n","\t"), '', $css);
		$this->ajax = 'false';

		// give plugins/themes lots of time to set NO_SmackQuery_DISPLAY constant
		add_action('init', array($this, 'hook_setup'), 9999);

		// hooks for things where usage display should be suppressed
		add_filter('wp_xmlrpc_server_class', array($this, 'no_query_display'));
		add_filter('rest_send_nocache_headers', array($this, 'no_query_display'));
	}

	function hook_setup() {
		// allow a theme/plugin to disable SmackQuery display
		if (!defined('NO_SmackQuery_DISPLAY')) {
			if (defined('WP_ADMIN')) {
				add_action('admin_init', array($this, 'smack_time_to_first_byte'), 9999);
				add_action('admin_footer', function(){register_shutdown_function(array($this, 'query_usage'));}, 9999);
			}
			else {
				add_action('wp_loaded', array($this, 'smack_time_to_first_byte'), 9999);
				add_action('wp_footer', function(){register_shutdown_function(array($this, 'query_usage'));}, 9999);
			}
		}
	}

	// function works for filter or action hooks
	function no_query_display($val) {
		$this->display = false;
		return $val;
	}

	function smack_time_to_first_byte() {
		// theme display/customizer does some, um, unusual things...
		if (is_admin()) {
			$this->css .= '.wrap .theme-overlay .theme-wrap,.wrap .theme-overlay .theme-backdrop,.wrap .wp-full-overlay-sidebar-content,#customize-preview iframe{bottom:24px}';
		}
		//$server_array = filter_input_array(INPUT_SERVER, FILTER_SANITIZE_STRING);
		// PHP 5.4+ provides this value
		if (!empty($_SERVER['REQUEST_TIME_FLOAT'])) {
			$this->starttime = $_SERVER['REQUEST_TIME_FLOAT'];
			$this->servertime = strval(round(microtime(true) - $this->starttime, 2)) . '<span class="query-ss"></span>';
		}
	}

	function query_usage() {
		if ($this->display && !defined('WP_INSTALLING') && (!defined('DOING_AJAX') || (defined('DOING_AJAX') && $this->ajax)) && current_user_can('update_core')) {
			global $wpdb;

			$precision = 0;
			$memory_usage = memory_get_peak_usage() / 1048576;
			if ($memory_usage < 10) {
				$precision = 2;
			}
			else if ($memory_usage < 100) {
				$precision = 1;
			}

			$memory_usage = round($memory_usage, $precision);
			$time_usage = (empty($this->starttime)) ? '' : $this->servertime . round(microtime(true) - $this->starttime, 2);
           
            echo ((defined('DOING_AJAX')) ? '' : $this->css . '</style><div id="query_dd_spacer"></div>') . '<div class="query-dd"' . ((defined('DOING_AJAX') && $this->ajax) ? '>' : ' id="query_dd">') . "Memory:{$memory_usage}M<span class=\"query-bs\">Time:{$time_usage}s</span>Queries:{$wpdb->num_queries}Q</div>";
		
			$class = array(
				'smack-no-js',
			);
	
			if ( did_action( 'wp_head' ) ) {
				$class[] = sprintf( 'smack-theme-%s', get_template() );
				$class[] = sprintf( 'smack-theme-%s', get_stylesheet() );
			}
	
			if ( ! is_admin_bar_showing() ) {
				$class[] = 'smack-peek';
			}
			echo '<script type="text/javascript">' . "\n\n";
			echo 'var smack = ' . json_encode( $json ) . ';' . "\n\n";
			echo '</script>' . "\n\n";

			echo '<div id="query-view" class="' . implode( ' ', array_map( 'esc_attr', $class ) ) . '" dir="ltr">';
				echo '<div id="smack-side-resizer" class="smack-resizer"></div>';
				echo '<div id="smack-title" class="smack-resizer">';
					echo '<h1 class="smack-title-heading">' . esc_html__( 'Smack Query / Asset View', 'all-in-one' ) . '</h1>';
					echo '<button class="smack-title-button smack-button-container-position" aria-label="' . esc_html__( 'Toggle panel position', 'all-in-one' ) . '"><span class="dashicons dashicons-image-rotate-left" aria-hidden="true"></span></button>';
					echo '<button class="smack-title-button smack-button-container-close" aria-label="' . esc_attr__( 'Close Panel', 'all-in-one' ) . '"><span class="dashicons dashicons-no-alt" aria-hidden="true"></span></button>';
				echo '</div>';
				$this->query_output_view();
				$this->asset_output_view();
			echo '</div>';
		}
	}

	public function query_output_view(){
		$db = get_option('data_db');				
		$this->query_row = 0;
		echo '<div id="smack-panels" class="smack_query_table_view" style="display:none">';
			echo '<div class="smack smack-panel-show">';
				echo '<table class="smack-sortable">';
					echo '<thead>';
						echo '<tr>';
							echo '<th scope="col" class="smack-sorted-asc smack-sortable-column" role="columnheader" aria-sort="ascending">';
								echo esc_html__( 'no', 'all-in-one' ); // WPCS: XSS ok;
							echo '</th>';
							echo '<th scope="col" class="smack-filterable-column">'; 
								echo '<div class="smack-filter-container">';
									echo '<label for="smack-query-view">'.esc_html__( 'Query') . '</label>';
									echo '<select id="smack-query-view" class="smack-filter"  data-filter="type">';
										echo '<option value="">' . esc_html_x( 'All', '"All" option for filters', 'all-in-one' ) . '</option>';
										echo '<option value="' . esc_attr( 'SHOW' ) . '">' . esc_html( 'SHOW' ) . '</option>';
										echo '<option value="' . esc_attr( 'SELECT' ) . '">' . esc_html( 'SELECT' ) . '</option>';
										echo '<option value="' . esc_attr( 'DELETE' ) . '">' . esc_html( 'DELETE' ) . '</option>';
										echo '<option value="' . esc_attr( 'CREATE' ) . '">' . esc_html( 'CREATE' ) . '</option>';
									echo '</select>';
								echo '</div>';
							echo '</th>';
							echo '<th scope="col">' . esc_html__( 'Callers', 'all-in-one' ) . '</th>';
							echo '<th scope="col">' . esc_html__( 'Components', 'all-in-one' ) . '</th>';
							echo '<th scope="col">' . esc_html__( 'rows', 'all-in-one' ) . '</th>';
							echo '<th scope="col">' . esc_html__( 'time', 'all-in-one' ) . '</th>';
						echo '</tr>';
					echo '</thead>';
					echo '<tbody>';
						foreach ( $db['dbs']['$wpdb']->rows as $row ) {
							$type    = $this->get_query_type( $row['sql'] );
							$sql_out = $this->format_sql( $row['sql'] );
							if ( 'SELECT' !== $type ) {
								$sql_out = "<span class='smack-nonselectsql'>{$sql_out}</span>";
							}
							echo '<tr data-smack-type="'.esc_attr($type).'">';
								echo '<td class="smack-row-num smack-num">';
									echo intval( ++$this->query_row );
								echo'</td>';
								echo '<td class="smack-row-sql smack-ltr smack-wrap">';
									echo $sql_out; // WPCS: XSS ok;
								echo '</td>';
								echo '<td>';
									echo $row['caller_name']; 
								echo '</td>';
								echo '<td>';
									echo $row['component']->name; 
								echo'</td>';
								echo '<td>';
									echo $row['result'];
								echo'</td>';
								echo '<td>';
									$stime = number_format_i18n( $row['ltime'], 4 );
									echo $stime;
								echo'</td>';
							echo '</tr>';
						}
					echo '</tbody>';
				echo '</table>';
			echo '</div>';
		echo '</div>';
	}

	public static function get_query_type( $sql ) {
		
		$sql = ltrim( $sql, ' \t\n\r\0\x0B(' );

		if ( 0 === strpos( $sql, '/*' ) ) {
			
			$sql = preg_replace( '|^/\*[^\*/]+\*/|', '', $sql );
		}

		$words = preg_split( '/\b/', trim( $sql ), 2, PREG_SPLIT_NO_EMPTY );
		$type = strtoupper( $words[0] );

		return $type;
	}
	
	public static function format_sql( $sql ) {

		$sql = str_replace( array( "\r\n", "\r", "\n", "\t" ), ' ', $sql );
		$sql = esc_html( $sql );
		$sql = trim( $sql );

		$regex = 'ADD|AFTER|ALTER|AND|BEGIN|COMMIT|CREATE|DELETE|DESCRIBE|DO|DROP|ELSE|END|EXCEPT|EXPLAIN|FROM|GROUP|HAVING|INNER|INSERT|INTERSECT|LEFT|LIMIT|ON|OR|ORDER|OUTER|RENAME|REPLACE|RIGHT|ROLLBACK|SELECT|SET|SHOW|START|THEN|TRUNCATE|UNION|UPDATE|USE|USING|VALUES|WHEN|WHERE|XOR';
		$sql   = preg_replace( '# (' . $regex . ') #', '<br> $1 ', $sql );

		$keywords = '\b(?:ACTION|ADD|AFTER|ALTER|AND|ASC|AS|AUTO_INCREMENT|BEGIN|BETWEEN|BIGINT|BINARY|BIT|BLOB|BOOLEAN|BOOL|BREAK|BY|CASE|COLLATE|COLUMNS?|COMMIT|CONTINUE|CREATE|DATA(?:BASES?)?|DATE(?:TIME)?|DECIMAL|DECLARE|DEC|DEFAULT|DELAYED|DELETE|DESCRIBE|DESC|DISTINCT|DOUBLE|DO|DROP|DUPLICATE|ELSE|END|ENUM|EXCEPT|EXISTS|EXPLAIN|FIELDS|FLOAT|FOREIGN|FOR|FROM|FULL|FUNCTION|GROUP|HAVING|IF|IGNORE|INDEX|INNER|INSERT|INTEGER|INTERSECT|INTERVAL|INTO|INT|IN|IS|JOIN|KEYS?|LEFT|LIKE|LIMIT|LONG(?:BLOB|TEXT)|MEDIUM(?:BLOB|INT|TEXT)|MERGE|MIDDLEINT|NOT|NO|NULLIF|ON|ORDER|OR|OUTER|PRIMARY|PROC(?:EDURE)?|REGEXP|RENAME|REPLACE|RIGHT|RLIKE|ROLLBACK|SCHEMA|SELECT|SET|SHOW|SMALLINT|START|TABLES?|TEXT(?:SIZE)?|THEN|TIME(?:STAMP)?|TINY(?:BLOB|INT|TEXT)|TRUNCATE|UNION|UNIQUE|UNSIGNED|UPDATE|USE|USING|VALUES?|VAR(?:BINARY|CHAR)|WHEN|WHERE|WHILE|XOR)\b';
		$sql      = preg_replace( '#' . $keywords . '#', '<b>$0</b>', $sql );

		return '<code>' . $sql . '</code>';

	}

	public function asset_output_view()
	{
		$page = home_url( $_SERVER['REQUEST_URI']);
        $page_value = parse_url($page);
        $current_page = $page_value['path'].$page_value['query'];
			echo '<div id="smack-wrapper">';
				echo '<nav id="smack-panel-menu" class="smack_asset_view" style="display:none;width:15%" aria-labelledby="smack-panel-menu-caption" style="width:15%">';
					echo '<ul role="tablist">';
						$core = get_option('smack_aio_assert_plugin_table');
						$plugin = get_option('smack_aio_assert_core_table');
						$total = array( "plugin"=> $core,"core"=> $plugin); 
						foreach($core as $core_plugin => $plugin){
							$children[] = array(
								"id" => $core_plugin,
								"href" => '#'.$core_plugin,
								"title" => $core_plugin
							);
						}

						$menus['plugin'] = array(
							"id"=>"plugin", 
							"href"=>"#plugin",
							"title"=>"plugin",
							"children"=>$children
						);

						$menus['core'] = array(
								"id"=>"core",
								"href"=>"#core",
								"title"=>"core"
						);

						foreach ( $menus as $menu ) {
							printf(
								'<li role="presentation"><button class="smack_button" role="tab" id="%1$s" data-smack-href="%2$s" value="%3$s">%4$s</button></li>',
								esc_attr( $menu['id'] ),
								esc_html( $menu['href'] ),
								esc_attr( $menu['href'] ),
								esc_attr( $menu['title'] )
							);
							$ids[] = $menu['id'];
							if ( ! empty( $menu['children'] ) ) {
								echo '<ul role="presentation">';
								foreach ( $menu['children'] as $child_id => $child ) {
									printf(
										'<li role="presentation"><button class="smack_button" role="tab" id="%1$s" data-smack-href="%2$s" value="%3$s">%4$s</button></li>',
										esc_attr( $child['id'] ),
										esc_html( $child['href'] ),
										esc_attr( $child['href'] ),
										esc_attr( $child['title'] )
									);
									$ids[] = $child['id'];
								}
								echo '</ul>';
							}
						}
					echo '</ul>';
				echo '</nav>'; 

				echo '<div id="smack-panels" class="smack_asset_view" style="display:none">';
					foreach($ids as $id){                
					echo '<div id="smack-panels-'.$id.'" class="smack-panels" style="display:none">';
						echo '<div class="smack">';
							printf('<div class="smack-panel-container" id="smack-%s-container">' ,esc_html( $id ));
								echo '<table class="smack-sortable">';
									echo '<thead>';
										echo '<tr>';
											echo '<th scope="col">' . esc_html__( 'source', 'all-in-one' ) . '</th>';
											echo '<th scope="col">' . esc_html__( 'type', 'all-in-one' ) . '</th>';    
											echo '<th scope="col">' . esc_html__( 'version', 'all-in-one' ) . '</th>';
											echo '<th scope="col">' . esc_html__( 'size', 'all-in-one' ) . '</th>';
											echo '<th scope="col">' . esc_html__( 'Action', 'all-in-one' ) . '</th>';
										echo '</tr>';
									echo '</thead>';
									echo '<tbody>';
									if($id == 'core'){
										$core = $total['core']; 
										foreach ( $core as $rows ) {
											foreach ( $rows as $row ) {
											echo '<tr>';
												echo '<td>';
													echo $row->script_name; 
												echo '</td>';
												echo '<td>';
													echo $row->type; 
												echo '</td>';
												echo '<td>';
													echo $row->version; 
												echo'</td>';
												echo '<td>';
													echo $row->size.' Kb';
												echo'</td>';
												echo '<td>';
													$checking = unserialize($row->current_page);
													foreach($checking as $url => $checked_value){
														if($url == $current_page){
															echo '<input class="data-script" type="checkbox" id="switch" data-url="'.$url.'" data-script="'.$row->script_name.'" data-type="core" data-scripttype="'.$row->type.'" style="float:none" value = "'.$checked_value.'" />';
														}
													}
												echo'</td>';
											echo '</tr>';
										}}  
									}elseif($id == 'plugin'){
										foreach($total['plugin'] as $plugin_name => $plugin_script ){
											foreach ( $plugin_script as $row ) {
												echo '<tr>';
													echo '<td>';
														echo $row->script_name; 
													echo '</td>';
													echo '<td>';
														echo $row->type; 
													echo '</td>';
													echo '<td>';
														echo $row->version; 
													echo'</td>';
													echo '<td>';
														echo $row->size.' Kb';
													echo'</td>';
													echo '<td>';
														$checking = unserialize($row->current_page);
														foreach($checking as $url => $checked_value){
															if($url == $current_page){
																echo '<input class="data-script" type="checkbox" id="switch" data-url="'.$url.'" data-script="'.$row->script_name.'" data-type="plugins" data-scripttype="'.$row->type.'" style="float:none" value = "'.$checked_value.'" />';
															}
														}
													echo'</td>';
												echo '</tr>';
											}
										}
									}
									else{
										foreach($total['plugin'] as $plugin_name => $plugin_script ){
											if($id == $plugin_name){
												$array = array($plugin_name=>$plugin_script);
												$core = $array;
												foreach ( $core as $rows ) {
													foreach ( $rows as $row ) {
													echo '<tr>';
														echo '<td>';
															echo $row->script_name; 
														echo '</td>';
														echo '<td>';
															echo $row->type; 
														echo '</td>';
														echo '<td>';
															echo $row->version; 
														echo'</td>';
														echo '<td>';
															echo $row->size.' Kb';
														echo'</td>';
														echo '<td>';
															$checking = unserialize($row->current_page);
															foreach($checking as $url => $checked_value){
																if($url == $current_page){
																	echo '<input class="data-script" type="checkbox" id="switch" data-url="'.$url.'" data-script="'.$row->script_name.'" data-type="plugins" data-scripttype="'.$row->type.'" style="float:none" value = "'.$checked_value.'" />';
																}
															}
														echo'</td>';
													echo '</tr>';
												}}  
											}else{
												$core = '';
											}
										}
									}
									echo '</tbody>';
								echo '</table>';
							echo '</div>';
						echo '</div>';
					echo "</div>";
					}
				echo '</div>';
			echo '</div>';
	}
}

add_action('plugins_loaded', function(){new SmackQuery();}, 1);
